header {* CAN and Nodes *}



theory CAN imports Lemmas LaTeXsugar
begin


subsection{* Definitions about CAN and Zones *}

consts max_node :: nat

consts degree:: nat (* degree of the CAN*)


typedef Node = "{n :: nat. n <= max_node }"
by auto;
(*Node is the set of nodes*)
typedef Degrees = "{n :: nat. n <= degree }"
by auto;

consts SIZE_Space :: nat
typedef Space = "{n :: nat. n <= SIZE_Space }"
by auto;


type_synonym Tuple = "Degrees\<Rightarrow>Space"

types Zone = "Tuple set" (* simpler and more efficient *)

definition Nat::"Space \<Rightarrow>nat"
where "Nat n == Rep_Space n"

definition intersects:: "Zone \<Rightarrow>Zone \<Rightarrow> bool"
where "intersects Z1 Z2 \<equiv> \<exists> tup::Tuple.  tup \<in> Z1 \<and> tup \<in> Z2"

definition NatNode::"Node \<Rightarrow>nat"
where "NatNode n == Rep_Node n"

lemma intersects_nonemptyD1: "intersects Z1 Z2 \<Longrightarrow> Z1\<noteq>{}"
  by (metis ex_in_conv intersects_def)


lemma intersects_nonemptyD2: "intersects Z1 Z2 \<Longrightarrow> Z2\<noteq>{}"
  by (metis ex_in_conv intersects_def)


lemma  everything_smallerthan_max: " Nat (n) \<le> ( SIZE_Space)"
proof - 
  have "Rep_Space n \<in> Space"
    by (metis Rep_Space)
  thus " Nat (n) \<le> ( SIZE_Space)"
    by (simp add: Space_def Nat_def Abs_Space_inverse)
  qed
    
text{*MARK*}
typedef CAN = "{(nodes::nat set, Z::nat=>Zone, neighbours::(nat\<times>nat) set).
                  finite nodes \<and> finite neighbours \<and> 
                  (\<forall> x y. (x,y)\<in>neighbours\<longrightarrow>(y,x)\<in>neighbours)\<and>
                  (\<forall> x. (x,x)\<notin>neighbours) \<and>
                  (\<forall>tup. \<exists>n\<in>nodes. tup\<in>(Z n)) \<and>
                  (\<forall> N\<in>nodes. \<forall> N'\<in>nodes. N\<noteq>N'\<longrightarrow>\<not>intersects (Z N) (Z N')) \<and>
                  (\<forall> N\<in>nodes. Z N\<noteq>{})}"
 -- "CAN DEFINITION"
apply auto;
apply (rule_tac x="{0}" in exI)
apply auto;
apply (rule_tac x="\<lambda>n.{tup. (\<forall> d. 0 \<le> Rep_Space(tup d) \<and>Rep_Space(tup d)\<le> SIZE_Space) }" in exI)
apply simp;
apply (subgoal_tac "\<exists>b\<Colon>nat \<times> nat \<Rightarrow> bool. finite b \<and> (\<forall>x y. (x, y) \<in> b \<longrightarrow> (y, x) \<in> b)\<and>
                  (\<forall> x. (x,x)\<notin> b)");
defer;apply force;
apply clarify;
apply (rule_tac x=b in exI);
apply simp+;
apply rule;
apply clarify;

(*2 *)
apply (fold Nat_def, rule everything_smallerthan_max) (*to show n\<le>MAX !*)

apply (rule_tac x="\<lambda>n.(Abs_Space 0)" in exI)
apply (intro allI,rule everything_smallerthan_max)
done
text{*
\begin{center}
@{thm CAN_def}
\end{center}
*}

definition  CAN_Nodes:: "CAN \<Rightarrow>nat set"
where "CAN_Nodes C \<equiv> (\<lambda>(x,y,z).x) (Rep_CAN C)"
definition  CAN_Zones:: "CAN \<Rightarrow>nat\<Rightarrow>Zone"
where "CAN_Zones C N\<equiv> ((\<lambda>(x,y,z).y) (Rep_CAN C)) N"
definition  CAN_neighbour:: "CAN \<Rightarrow>nat \<Rightarrow>nat \<Rightarrow> bool"
where "CAN_neighbour C N N' \<equiv> (N,N')\<in>((\<lambda>(x,y,z).z) (Rep_CAN C))"

abbreviation destination_NL:: "nat list \<Rightarrow> nat"
where "destination_NL NL \<equiv> NL!(length NL - 1)"

abbreviation NodesInZone:: "CAN \<Rightarrow> Zone \<Rightarrow> nat \<Rightarrow> bool" 
where
"NodesInZone C Z \<equiv>  {N\<in>CAN_Nodes C. intersects (CAN_Zones C N) Z}"

text{*MARK*}
definition Connected:: "CAN\<Rightarrow>Zone \<Rightarrow>bool" where 
"Connected C Z\<equiv>\<forall>n\<in>NodesInZone C Z. \<forall>n'\<in>NodesInZone C Z.
  \<exists> node_list. node_list\<noteq>[]\<and> node_list!0=n \<and> destination_NL node_list=n' \<and> distinct node_list
         \<and> (\<forall> i<length node_list - 1. node_list!i\<in>CAN_Nodes C
              \<and> CAN_neighbour C (node_list!i) (node_list!(i+1)) 
              \<and> node_list!i\<in>NodesInZone C Z)"

lemma finite_intersects: "finite (NodesInZone C Z)"
proof -
have "Rep_CAN C\<in>CAN"
  by (rule  Rep_CAN)
thus ?thesis
  unfolding CAN_Nodes_def CAN_Zones_def CAN_def
  by (force)
qed

lemma finite_intersects_2: "finite {node\<in>(CAN_Nodes C). intersects Z (CAN_Zones C node) }"
proof -
have "Rep_CAN C\<in>CAN"
  by (rule  Rep_CAN)
thus ?thesis
  unfolding CAN_Nodes_def CAN_Zones_def CAN_def
  by (force)
qed

lemma zone_non_empty: "N\<in>CAN_Nodes C \<Longrightarrow> CAN_Zones C N\<noteq>{}"
proof -
assume  h: "N\<in>CAN_Nodes C"
have "Rep_CAN C\<in>CAN" by (rule  Rep_CAN)
with h show ?thesis
unfolding CAN_Nodes_def CAN_Zones_def CAN_def  by (force)
qed
(*
apply (simp add: CAN_def)
apply (force simp: CAN_Nodes_def CAN_Zones_def)
apply (rule Rep_CAN);
done
*)

lemma intersects_non_empty: "intersects Z Z' = (Z\<inter>Z'\<noteq> {})"
by (auto simp: intersects_def);

lemma intersect_sym: "intersects Z Z' \<Longrightarrow>intersects Z' Z"
by (auto simp: intersects_def);

lemma finite_R: "finite (R::(nat\<times>nat) set)\<Longrightarrow>finite {y. (x,y)\<in>R}"
proof -
assume h: "finite (R::(nat\<times>nat) set)"
with  image_Collect have "finite {(x,y). (x,y)\<in>R}" by auto
 hence "finite (snd ` {(x,y). (x,y)\<in>R})"..
moreover   
have  "{y. (x, y) \<in> R} \<subseteq> (snd ` {(x, y). (x, y) \<in> R})" unfolding image_Collect by auto
ultimately show ?thesis
   proof  (rule_tac  finite_subset) qed
qed
(*apply  (subgoal_tac "finite {(x,y). (x,y)\<in>R}");
apply (rotate_tac 1);
apply (frule_tac h=snd in finite_imageI);
apply (simp only: image_Collect)
apply simp;
apply (rule finite_subset)
apply auto;
done;*)

lemma finite_neighbours: "finite {N'. CAN_neighbour C N N'}"
proof -
  have "Rep_CAN C\<in> CAN" by (rule Rep_CAN)
  thus ?thesis
    unfolding CAN_neighbour_def CAN_def
    using finite_R[of "(\<lambda>(x, y, z). z) (Rep_CAN C)" N]
    by auto
  qed
(*  
apply (unfold CAN_neighbour_def)
apply (subgoal_tac "Rep_CAN C\<in> CAN");
apply (simp add: CAN_def)
apply auto;
apply (rule finite_R);
apply assumption;
apply (simp add: Rep_CAN);
done
*)
lemma intersects_empty: "(NodesInZone C Z={}) \<Longrightarrow>  Z={}"
  (is "?NodesEmpty \<Longrightarrow> ?Zempty")
proof -
assume ?NodesEmpty
have repcan: "Rep_CAN C\<in>CAN" by (rule Rep_CAN)
with `?NodesEmpty` have "(Z\<noteq>{}) \<longrightarrow> False"
  proof (intro impI)
    assume "Z\<noteq>{}"
    with repcan and `?NodesEmpty` show False 
      proof -
        from `Z\<noteq>{}` obtain tup where "tup\<in>Z" by auto
        with repcan have "\<exists>node\<in>CAN_Nodes C. intersects Z (CAN_Zones C node)" 
          proof -
            from repcan obtain No and Zo and Ne where C: "Rep_CAN C = (No,Zo,Ne)" unfolding CAN_def by auto
            with  repcan  `tup\<in>Z` have "\<exists> node\<in>No. tup \<in> Zo node"  by (auto simp: CAN_def CAN_Nodes_def CAN_Zones_def)
            with  `tup\<in>Z` C show ?thesis by (auto simp: CAN_def intersects_def CAN_Nodes_def CAN_Zones_def)
            qed
         with `?NodesEmpty` show ?thesis by (auto simp: intersects_def)
      qed
    qed
thus "Z={}" by auto
qed
(*apply auto;
apply (unfold empty_def CAN_Zones_def intersects_def);
apply auto;
apply (subgoal_tac "Rep_CAN C\<in>CAN")
defer; apply (rule Rep_CAN);
(*1*)
apply (unfold CAN_def)
apply clarify;
apply auto;
apply (drule_tac x=x in spec);
apply clarify;
apply (drule_tac x=n in spec);
apply auto;
apply (subgoal_tac "Rep_CAN C\<in>CAN")
apply (force simp: CAN_Nodes_def);
apply (rule Rep_CAN);
done;
*)

lemma intersects_empty_2: "({node\<in>(CAN_Nodes C). intersects Z (CAN_Zones C node)}={}) \<Longrightarrow>  Z={}"
  (is "?NodesEmpty \<Longrightarrow> ?Zempty")
proof -
assume ?NodesEmpty
hence "NodesInZone C Z={}" by (auto simp: intersects_def CAN_Nodes_def)
thus ?thesis by (rule intersects_empty)
qed

lemma intersects_empty_rev: "tup\<in>Z \<Longrightarrow> (\<exists>node\<in>(CAN_Nodes C). intersects Z (CAN_Zones C node)\<and>tup\<in> CAN_Zones C node)"
proof -
assume "tup\<in>Z"
  moreover have repcan: "Rep_CAN C\<in> CAN" by (rule Rep_CAN)
  moreover then obtain No and Zo and Ne where "Rep_CAN C = (No,Zo,Ne)" unfolding CAN_def by auto
  moreover with repcan  `tup\<in>Z` have "\<exists> node\<in>No. tup \<in> Zo node"  by (auto simp: CAN_def CAN_Nodes_def CAN_Zones_def)
  ultimately show  ?thesis
    by (auto simp: intersects_def CAN_Zones_def CAN_Nodes_def)
qed
(*
apply (subgoal_tac "Rep_CAN C\<in>CAN")
defer; apply (rule Rep_CAN);
apply (unfold CAN_def)
apply clarify;
apply (drule_tac x=tup in spec);
apply (auto simp: intersects_def CAN_Zones_def CAN_Nodes_def);
done;
*)



lemma inc_single_zone:  "\<lbrakk>N\<in> CAN_Nodes C; N' \<in> CAN_Nodes C; Z \<subseteq> CAN_Zones C N; N'\<in>NodesInZone C Z \<rbrakk> \<Longrightarrow>N=N'"
(is "\<lbrakk>?ncannodes;?npcannodes;?ZZone;?intersectsZ\<rbrakk>\<Longrightarrow>?ccl")
proof -
  assume ?ncannodes ?npcannodes ?ZZone ?intersectsZ
  moreover have repcan: "Rep_CAN C\<in> CAN" by (rule Rep_CAN)
  moreover then obtain No and Zo and Ne where "Rep_CAN C = (No,Zo,Ne)" unfolding CAN_def by auto
  moreover with  repcan `?ncannodes` `?npcannodes` 
        have "N \<noteq> N' \<longrightarrow> \<not> intersects (Zo N) (Zo N')" by (auto simp: CAN_Nodes_def CAN_Zones_def CAN_def)
  ultimately show ?thesis by (auto simp: intersects_def CAN_Zones_def)
qed
(*
apply (unfold intersects_def CAN_Zones_def CAN_Nodes_def);
apply auto;
apply (subgoal_tac "Rep_CAN C \<in> CAN")
apply (simp add: CAN_def);
apply clarify;
apply (drule_tac x=N in bspec);
apply simp+;
apply (drule_tac x=N' in bspec);
apply (simp add: intersects_def)+;
apply blast;
apply (rule Rep_CAN);
done;
*)

lemma inc_single_zone_2:  "\<lbrakk>N\<in> CAN_Nodes C; N' \<in> CAN_Nodes C; intersects (CAN_Zones C N) (CAN_Zones C N') \<rbrakk> \<Longrightarrow>N=N'"
by (drule_tac Z="(CAN_Zones C N')" in inc_single_zone, auto)

lemma CAN_covered_diff_node: "x\<notin>CAN_Zones C N \<Longrightarrow>\<exists> N'\<in> CAN_Nodes C. x\<in>CAN_Zones C N'\<and> N\<noteq> N'"
proof -
  assume "x\<notin>CAN_Zones C N"
  moreover have repcan: "Rep_CAN C\<in> CAN" by (rule Rep_CAN)
  moreover then obtain No and Zo and Ne where Cdef: "Rep_CAN C = (No,Zo,Ne)" unfolding CAN_def by auto
  moreover with  repcan have "\<exists>n\<in>No. x \<in> Zo n" by (auto simp: CAN_Nodes_def CAN_Zones_def CAN_def)
  then obtain N' where Np: "x \<in> Zo N'" "N'\<in>No".. 
  moreover with `x\<notin>CAN_Zones C N` repcan Cdef have "N\<noteq>N'" 
    proof cases
      assume "N\<in>CAN_Nodes C" with Np repcan Cdef
        have "N \<noteq> N' \<longrightarrow> \<not> intersects (Zo N) (Zo N')" by (auto simp: CAN_Nodes_def CAN_Zones_def CAN_def)
        with Np Cdef `x\<notin>CAN_Zones C N` show ?thesis by (auto simp: intersects_def CAN_Zones_def CAN_Nodes_def)
      next
        assume "N\<notin>CAN_Nodes C"
        with Np Cdef show ?thesis by (auto simp: CAN_Nodes_def)
      qed
  ultimately show ?thesis by (auto simp: intersects_def CAN_Zones_def CAN_Nodes_def)
qed
(*apply (unfold  CAN_Zones_def CAN_Nodes_def);
apply (subgoal_tac "Rep_CAN C \<in> CAN")
apply (simp add: CAN_def);
apply clarify;
apply auto;
apply (rule Rep_CAN);
done;*)

lemma Multiple_Zones: "\<lbrakk>N\<in> CAN_Nodes C;\<not>Z \<subseteq> CAN_Zones C N\<rbrakk> \<Longrightarrow>\<exists> N' \<in>NodesInZone C Z. N'\<noteq>N "
  (is "\<lbrakk> ?nnode;?ZZone\<rbrakk>\<Longrightarrow>_")
proof -
assume ?nnode ?ZZone
hence "\<exists>x. x\<in>Z \<and>x\<notin> CAN_Zones C N" by (auto simp: intersects_def)
then obtain x where  "x\<in>Z" "x\<notin> CAN_Zones C N" by auto
with CAN_covered_diff_node have "\<exists> N'\<in> CAN_Nodes C. x\<in>CAN_Zones C N'\<and> N\<noteq> N'" by auto
then obtain N' where  "N'\<in> CAN_Nodes C" "x\<in>CAN_Zones C N'" "N\<noteq> N'" by auto
with `x\<in>Z` show ?thesis by (auto simp: intersects_def)
qed
(*apply (auto);
apply (rule ccontr);
apply (frule CAN_different);
apply clarify;
apply (drule_tac x=N' in bspec);
apply simp;
apply (subgoal_tac " (N\<noteq> N' \<and> intersects (CAN_Zones C N') Z) \<longrightarrow> \<not>Z \<subseteq> CAN_Zones C N")
apply (simp add: intersects_def)+;
done
*)

lemma intersect_included: "\<lbrakk>Z\<subseteq> Z'; intersects Z'' Z\<rbrakk>\<Longrightarrow>intersects Z'' Z'"
by (auto simp: intersects_def);

lemma finite_CAN_Nodes: "finite (CAN_Nodes C)"
proof -
 have repcan: "Rep_CAN C\<in> CAN" by (rule Rep_CAN)
 thus ?thesis by (auto simp: CAN_def CAN_Nodes_def)
qed
(*apply (subgoal_tac "Rep_CAN C \<in> CAN");
apply (simp add: CAN_def CAN_Nodes_def);
apply auto;
apply (rule Rep_CAN);
done;*)


lemma neighbour_diff: "CAN_neighbour C N N' \<Longrightarrow> N\<noteq> N'"
proof
  assume "CAN_neighbour C N N'" "N=N'"
  moreover have repcan: "Rep_CAN C\<in> CAN" by (rule Rep_CAN)
  ultimately show False by  (auto simp: CAN_def CAN_Nodes_def CAN_neighbour_def)
qed
(*apply (unfold CAN_neighbour_def) 
apply (subgoal_tac "Rep_CAN C \<in> CAN");
apply (simp add: CAN_def CAN_Nodes_def);
apply force;
apply (rule Rep_CAN);
done; *)

lemma neighbour_refl: "CAN_neighbour C N N' \<Longrightarrow> CAN_neighbour C N' N "
proof -
  assume "CAN_neighbour C N N'" 
  moreover have repcan: "Rep_CAN C\<in> CAN" by (rule Rep_CAN)
  ultimately show ?thesis by  (auto simp: CAN_def CAN_Nodes_def CAN_neighbour_def)
qed

lemma disjoint_zones: "\<lbrakk>N\<in>CAN_Nodes C; N'\<in>CAN_Nodes C; N\<noteq>N'\<rbrakk> \<Longrightarrow>\<not> intersects (CAN_Zones C N) (CAN_Zones C N')"
proof
  assume "N \<in> CAN_Nodes C" "N' \<in> CAN_Nodes C" "N \<noteq> N'" "intersects (CAN_Zones C N) (CAN_Zones C N')"
  moreover have repcan: "Rep_CAN C\<in> CAN" by (rule Rep_CAN)
  ultimately show False by  (auto simp: CAN_def CAN_Nodes_def CAN_Zones_def)
qed
  (*apply (subgoal_tac "Rep_CAN C \<in> CAN")
apply (force simp:  CAN_def CAN_Zones_def CAN_Nodes_def) 
apply (rule Rep_CAN)
done*)

subsection{* NodesInZone *}


abbreviation NoNodeInBothZones:: "CAN \<Rightarrow> Zone \<Rightarrow> Zone \<Rightarrow> bool"
where
"NoNodeInBothZones C Z1 Z2 \<equiv>  {N\<in>CAN_Nodes C. intersects (CAN_Zones C N) Z1}\<inter> {N\<in>CAN_Nodes C. intersects (CAN_Zones C N) Z2} = {}"

abbreviation ZoneSize::"CAN \<Rightarrow> Zone \<Rightarrow> nat"
  where "ZoneSize C Z \<equiv> card (NodesInZone C Z)"

lemma NodesInZone_exI: "x\<in>Z \<Longrightarrow> \<exists> N\<in> NodesInZone C Z. x\<in>(CAN_Zones C N)"
proof (simp only: intersects_def)
  assume "x\<in>Z"
  moreover have repcan: "Rep_CAN C\<in> CAN" by (rule Rep_CAN)
  moreover then obtain No and Zo and Ne where Cdef: "Rep_CAN C = (No,Zo,Ne)" unfolding CAN_def by auto
  moreover with repcan have " \<exists>n\<in>No. x \<in> Zo n" by (auto simp: CAN_def )
  then obtain N where "N\<in>No""x\<in>Zo N"..  
  ultimately show "\<exists>N\<in>{N \<in> CAN_Nodes C. \<exists>tup. tup \<in> CAN_Zones C N \<and> tup \<in> Z}. x \<in> CAN_Zones C N" 
    by (auto simp: CAN_Nodes_def CAN_Zones_def)
qed
(*apply (subgoal_tac "Rep_CAN C \<in> CAN");
apply (simp add: CAN_def CAN_Nodes_def CAN_Zones_def intersects_def);
apply clarify;
apply (drule_tac x=x in spec);
apply clarify;
apply (rule_tac x=n in exI);
apply force;
apply (rule Rep_CAN);
done;*)


lemma NodesInZoneI: "\<lbrakk>x\<in>Z; N\<in>CAN_Nodes C;  x\<in>(CAN_Zones C N)\<rbrakk>\<Longrightarrow> N\<in> NodesInZone C Z"
proof -
  assume "x\<in>Z" "N\<in>CAN_Nodes C"  "x\<in>(CAN_Zones C N)"
  moreover have repcan: "Rep_CAN C\<in> CAN" by (rule Rep_CAN)
  ultimately show ?thesis by (auto simp: intersects_def)
qed
(*apply (subgoal_tac "Rep_CAN C \<in> CAN");
apply (auto simp: intersects_def);
apply (rule Rep_CAN);
done;*)

text{*MARK*}
lemma NodesInZone_substract[intro]:
"\<lbrakk>N\<in>NodesInZone C Z; N'\<in>CAN_Nodes C; N\<noteq>N'\<rbrakk>
      \<Longrightarrow> N\<in>NodesInZone C (Z-CAN_Zones C N')"
by (drule disjoint_zones, auto simp: intersects_def)

lemma NodesInZoneInc: "Z\<subseteq>Z' \<Longrightarrow> NodesInZone C Z \<subseteq> NodesInZone C Z'"
by (auto simp: intersects_def) 

subsection{* Entire_Space *}
abbreviation Entire_Space:: Zone
where "Entire_Space \<equiv> {tup. (\<forall> d. 0 \<le> Rep_Space(tup d) \<and>Rep_Space(tup d)\<le> SIZE_Space) }"

lemma Entire_space_nodes:
"N\<in> CAN_Nodes C \<Longrightarrow>N\<in>NodesInZone C Entire_Space "
proof
assume "N\<in> CAN_Nodes C"
moreover with zone_non_empty have "\<exists> x. x\<in>(CAN_Zones C N)" by auto
then obtain x where "x\<in>(CAN_Zones C N)"..
moreover have "x\<in>Entire_Space"
proof (simp,intro allI)
  fix d::Degrees
  have "Rep_Space (x d)\<in>Space" by (metis Rep_Space)
  thus  "Rep_Space (x d) \<le> SIZE_Space"  by (auto simp: Space_def)
qed
ultimately show "N\<in> CAN_Nodes C \<and> intersects (CAN_Zones C N) Entire_Space" by (auto simp: intersects_def)
qed
(*apply (frule zone_non_empty)
apply (auto simp: intersects_def)
apply (drule_tac x=x in spec)
apply auto
apply (subgoal_tac "Rep_Space (x d)\<in>Space")
apply (simp add: Space_def Nat_def Abs_Space_inverse)
apply (rule Rep_Space)
done*)


lemma NodesInZoneUnion: "\<lbrakk>N\<in>NodesInZone C (Z\<union>Z') ; N\<notin>NodesInZone C Z\<rbrakk> \<Longrightarrow>N\<in>NodesInZone C Z'" by (auto simp: intersects_def)

lemma NodesInZoneSingleton[intro]:"NodesInZone C Z = {N}\<Longrightarrow>Z\<subseteq>CAN_Zones C N"
proof
  fix x
  assume "NodesInZone C Z = {N}" "x \<in> Z"
  thus  "x \<in> CAN_Zones C N"
    proof (rule_tac  ccontr)
      assume "NodesInZone C Z = {N}"" x \<in> Z"" x \<notin> CAN_Zones C N"
      from `x \<notin> CAN_Zones C N` have "\<exists> N'\<in> CAN_Nodes C. x\<in>CAN_Zones C N'\<and> N\<noteq> N'" by (rule CAN_covered_diff_node)
      then obtain N' where "N'\<in> CAN_Nodes C""x\<in>CAN_Zones C N'"" N\<noteq> N'" by auto
      with `x \<in> Z` have "N'\<in> NodesInZone C Z" by (auto simp:intersects_def)
      with `NodesInZone C Z = {N}` `N\<noteq>N'` show False by auto
    qed
  qed



lemma NoNodeInBothZones_disjoint: "NodesInZone C Z1 \<inter> NodesInZone C Z2 = {} \<Longrightarrow> Z1\<inter>Z2={}"
apply auto
apply (drule_tac C=C in NodesInZone_exI)+
apply auto
apply (subgoal_tac "Rep_CAN C \<in> CAN")
apply (simp add:  CAN_def CAN_Zones_def CAN_Nodes_def) 
apply auto
apply (drule_tac x=N in bspec)
apply simp
apply (rotate_tac -1, drule_tac x=Na in bspec)
apply simp
apply (auto simp: intersects_def)
apply (rule Rep_CAN)
done

lemma ZoneSize_included: "NodesInZone C Z\<subset>NodesInZone C Z' \<Longrightarrow> ZoneSize C Z < ZoneSize C Z'"
apply (subgoal_tac "finite (NodesInZone C Z')")
apply (rule Finite_Set.psubset_card_mono)
apply simp+
apply (insert finite_CAN_Nodes)
apply force
done

end